<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Storelocator\Controller\PickUpInStore;

use \Magento\Framework\Exception\NotFoundException;

/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class RmareceivedAction extends \Magento\Framework\App\Action\Action
{
    protected $jsonResultFactory;
    protected $helper;
    protected $_rma;
    protected $_rmaHistory;
    protected $_transportBuilder;
    protected $_storeManager;
    protected $_inlineTranslation;
    protected $_paymentHelper;
    protected $_addressRenderer;


    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \FiloBlu\Storelocator\Helper\Data $helper,
        \Magento\Rma\Model\Rma $rma,
        \Magento\Rma\Model\Rma\Status\History $rmaHistory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        $this->_rma = $rma;
        $this->_rmaHistory = $rmaHistory;
        $this->date = $date;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        parent::__construct($context);
    }


    public function execute()
    {

      $params = $this->getRequest()->getParams();

        if (!$this->helper->getConfig('carriers/pickupinstore/active') || !isset($params["rma_id"]) || !$params["rma_id"] || !isset($params["hash"]) || !$params["hash"]) {
            return $this->_forward('noRoute');
        }

        $this->_rma->load($params["rma_id"]);

        if ($this->_rma->getId() && $this->_rma->getReturnToStoreHash() == $params["hash"] && (int)$this->_rma->getReturnToStoreId() > 0) {
          $this->_rma->setReturnToStoreReceivedDate($this->date->gmtDate());
          $this->_rma->save();
          $resultJson = $this->resultJsonFactory->create();
          $resultJson->setData(["message" =>__("You confirmed that the package ") . $this->_rma->getData("increment_id") . __(" was accepted.")]);
          $this->updateRma();

          $this->_eventManager->dispatch('filoblu_storelocator_pickupinstore_rma_received', ['rma' => $this->_rma]);

          return $resultJson;
        }
        else {
          return $this->_forward('noRoute');
        }
    }

    public function updateRma(){

        $this->_rma->setStatus("authorized")->save();

        $comment = 'Return In Store - The package was picked up by store';
        $this->_rmaHistory->setRmaEntityId($this->_rma->getId());
        $this->_rmaHistory->setComment($comment);
        $this->_rmaHistory->saveComment($comment, false, true);

    }
}
