<?php

namespace FiloBlu\Storelocator\Setup;

use Exception;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Status;
use Magento\Sales\Model\Order\StatusFactory;
use Magento\Sales\Model\ResourceModel\Order\Status as StatusResource;
use Magento\Sales\Model\ResourceModel\Order\StatusFactory as StatusResourceFactory;

class UpgradeData implements UpgradeDataInterface {


    /**
     * Delivered in Store Order-Status code
     */
    const ORDER_STATUS_PROCESSING_CODE = 'delivered_in_store';

    /**
     * Delivered in Store Order-Status label
     */
    const ORDER_STATUS_PROCESSING_LABEL = 'Delivered in Store';

    /**
     * @var StatusFactory
     */
    protected $statusFactory;

    /**
     * @var StatusResourceFactory
     */
    protected $statusResourceFactory;

    /**
     * UpgradeData constructor.
     * @param StatusFactory $statusFactory
     * @param StatusResourceFactory $statusResourceFactory
     */
    public function __construct(
        StatusFactory $statusFactory,
        StatusResourceFactory $statusResourceFactory
    ) {
        $this->statusFactory = $statusFactory;
        $this->statusResourceFactory = $statusResourceFactory;
    }

    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context) {

        $setup->startSetup();

        if (version_compare($context->getVersion(), '1.2.0') < 0) {

            // Quote
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("quote"), 'pickup_in_store_id', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        'comment' => 'PickUp In Store ID'
                            ]
            );
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("quote"), 'pickup_in_store_email', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 100,
                        'nullable' => true,
                        'comment' => 'PickUp In Store Email'
                            ]
            );

            // Order
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("sales_order"), 'pickup_in_store_id', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        'comment' => 'PickUp In Store ID'
                            ]
            );
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("sales_order"), 'pickup_in_store_email', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 100,
                        'nullable' => true,
                        'comment' => 'PickUp In Store Email'
                            ]
            );

        }

        if (version_compare($context->getVersion(), '1.2.2') < 0) {

            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("sales_order"), 'pickup_in_store_confirm_date', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                        'nullable' => true,
                        'comment' => 'PickUp In Store Confirm Date'
                            ]
            );

        }

        if (version_compare($context->getVersion(), '1.2.3') < 0) {

            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable("sales_order"), 'pickup_in_store_hash', [
                              'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                              'length' => 100,
                              'nullable' => true,
                              'comment' => 'PickUp In Store Hash'
                            ]
            );

        }

        if (version_compare($context->getVersion(), '1.3.0') < 0) {

            $setup->getConnection()
                ->addColumn(
                    $setup->getTable("sales_order"), 'pickup_in_store_confirm_received_date', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                        'nullable' => true,
                        'comment' => 'PickUp In Store Confirm Received Date'
                    ]
                );

        }

        $this->addNewOrderProcessingStatus();

        $setup->endSetup();
    }

    /**
     * Create new order processing status
     *
     * @return void
     *
     * @throws Exception
     */
    protected function addNewOrderProcessingStatus()
    {
        /** @var StatusResource $statusResource */
        $statusResource = $this->statusResourceFactory->create();
        /** @var Status $status */
        $status = $this->statusFactory->create();
        $status->setData([
            'status' => self::ORDER_STATUS_PROCESSING_CODE,
            'label' => self::ORDER_STATUS_PROCESSING_LABEL,
        ]);

        try {
            $statusResource->save($status);
        } catch (AlreadyExistsException $exception) {

            return;
        }

        $status->assignState(Order::STATE_PROCESSING, false, true);
    }

}
